@@ -1,6 +1,6 @@
-/* the Music Player Daemon (MPD)
- * Copyright (C) 2003-2007 by Warren Dukes (warren.dukes@gmail.com)
- * This project's homepage is: http://www.musicpd.org
+/*
+ * Copyright (C) 2003-2009 The Music Player Daemon Project
+ * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -11,34 +11,46 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include "client.h"
+#include "fifo_buffer.h"
 #include "command.h"
 #include "conf.h"
-#include "log.h"
 #include "listen.h"
+#include "socket_util.h"
 #include "permission.h"
-#include "utils.h"
-#include "ioops.h"
-#include "main_notify.h"
-#include "dlist.h"
+#include "event_pipe.h"
 #include "idle.h"
-
-#include "../config.h"
+#include "main.h"
+#include "config.h"
 
 #include <glib.h>
 #include <assert.h>
+#include <unistd.h>
+#include <string.h>
+#include <stdlib.h>
+#include <stdio.h>
+#include <errno.h>
+
+#ifdef WIN32
+#include <ws2tcpip.h>
+#include <winsock.h>
+#else
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#include <unistd.h>
-#include <string.h>
+#endif
 
-#define GREETING				"OK MPD " PROTOCOL_VERSION "\n"
+#undef G_LOG_DOMAIN
+#define G_LOG_DOMAIN "client"
+#define LOG_LEVEL_SECURE G_LOG_LEVEL_INFO
+
+static const char GREETING[] = "OK MPD " PROTOCOL_VERSION "\n";
 
 #define CLIENT_LIST_MODE_BEGIN			"command_list_begin"
 #define CLIENT_LIST_OK_MODE_BEGIN			"command_list_ok_begin"
@@ -50,11 +62,9 @@
 
 /* set this to zero to indicate we have no possible clients */
 static unsigned int client_max_connections;	/*CLIENT_MAX_CONNECTIONS_DEFAULT; */
-static int client_timeout = CLIENT_TIMEOUT_DEFAULT;
-static size_t client_max_command_list_size =
-    CLIENT_MAX_COMMAND_LIST_DEFAULT;
-static size_t client_max_output_buffer_size =
-    CLIENT_MAX_OUTPUT_BUFFER_SIZE_DEFAULT;
+static int client_timeout;
+static size_t client_max_command_list_size;
+static size_t client_max_output_buffer_size;
 
 struct deferred_buffer {
 	size_t size;
@@ -62,19 +72,22 @@ struct deferred_buffer {
 };
 
 struct client {
-	struct list_head siblings;
+	GIOChannel *channel;
+	guint source_id;
 
-	char buffer[4096];
-	size_t bufferLength;
-	size_t bufferPos;
+	/** the buffer for reading lines from the #channel */
+	struct fifo_buffer *input;
 
-	int fd;	/* file descriptor; -1 if expired */
 	unsigned permission;
 
 	/** the uid of the client process, or -1 if unknown */
 	int uid;
 
-	time_t lastTime;
+	/**
+	 * How long since the last activity from this client?
+	 */
+	GTimer *last_activity;
+
 	GSList *cmd_list;	/* for when in list mode */
 	int cmd_list_OK;	/* print OK after each command execution */
 	size_t cmd_list_size;	/* mem cmd_list consumes */
@@ -96,16 +109,22 @@ struct client {
 	unsigned idle_subscriptions;
 };
 
-static LIST_HEAD(clients);
+static GList *clients;
 static unsigned num_clients;
+static guint expire_source_id;
 
 static void client_write_deferred(struct client *client);
 
 static void client_write_output(struct client *client);
 
+static void client_manager_expire(void);
+
+static gboolean
+client_in_event(GIOChannel *source, GIOCondition condition, gpointer data);
+
 bool client_is_expired(const struct client *client)
 {
-	return client->fd < 0;
+	return client->channel == NULL;
 }
 
 int client_get_uid(const struct client *client)
@@ -123,11 +142,32 @@ void client_set_permission(struct client *client, unsigned permission)
 	client->permission = permission;
 }
 
+/**
+ * An idle event which calls client_manager_expire().
+ */
+static gboolean
+client_manager_expire_event(G_GNUC_UNUSED gpointer data)
+{
+	expire_source_id = 0;
+	client_manager_expire();
+	return false;
+}
+
 static inline void client_set_expired(struct client *client)
 {
-	if (client->fd >= 0) {
-		xclose(client->fd);
-		client->fd = -1;
+	if (expire_source_id == 0 && !client_is_expired(client))
+		/* delayed deletion */
+		expire_source_id = g_idle_add(client_manager_expire_event,
+					      NULL);
+
+	if (client->source_id != 0) {
+		g_source_remove(client->source_id);
+		client->source_id = 0;
+	}
+
+	if (client->channel != NULL) {
+		g_io_channel_unref(client->channel);
+		client->channel = NULL;
 	}
 }
 
@@ -139,11 +179,27 @@ static void client_init(struct client *client, int fd)
 
 	client->cmd_list_size = 0;
 	client->cmd_list_OK = -1;
-	client->bufferLength = 0;
-	client->bufferPos = 0;
-	client->fd = fd;
-	set_nonblocking(fd);
-	client->lastTime = time(NULL);
+
+#ifndef G_OS_WIN32
+	client->channel = g_io_channel_unix_new(fd);
+#else
+	client->channel = g_io_channel_win32_new_socket(fd);
+#endif
+	/* GLib is responsible for closing the file descriptor */
+	g_io_channel_set_close_on_unref(client->channel, true);
+	/* NULL encoding means the stream is binary safe; the MPD
+	   protocol is UTF-8 only, but we are doing this call anyway
+	   to prevent GLib from messing around with the stream */
+	g_io_channel_set_encoding(client->channel, NULL, NULL);
+	/* we prefer to do buffering */
+	g_io_channel_set_buffered(client->channel, false);
+
+	client->source_id = g_io_add_watch(client->channel,
+					   G_IO_IN|G_IO_ERR|G_IO_HUP,
+					   client_in_event, client);
+
+	client->input = fifo_buffer_new(4096);
+
 	client->cmd_list = NULL;
 	client->deferred_send = g_queue_new();
 	client->deferred_bytes = 0;
@@ -152,7 +208,7 @@ static void client_init(struct client *client, int fd)
 
 	client->permission = getDefaultPermissions();
 
-	xwrite(fd, GREETING, strlen(GREETING));
+	(void)write(fd, GREETING, sizeof(GREETING) - 1);
 }
 
 static void free_cmd_list(GSList *list)
@@ -178,12 +234,15 @@ deferred_buffer_free(gpointer data, G_GNUC_UNUSED gpointer user_data)
 static void client_close(struct client *client)
 {
 	assert(num_clients > 0);
-	assert(!list_empty(&clients));
-	list_del(&client->siblings);
+	assert(clients != NULL);
+
+	clients = g_list_remove(clients, client);
 	--num_clients;
 
 	client_set_expired(client);
 
+	g_timer_destroy(client->last_activity);
+
 	if (client->cmd_list) {
 		free_cmd_list(client->cmd_list);
 		client->cmd_list = NULL;
@@ -192,69 +251,37 @@ static void client_close(struct client *client)
 	g_queue_foreach(client->deferred_send, deferred_buffer_free, NULL);
 	g_queue_free(client->deferred_send);
 
-	SECURE("client %i: closed\n", client->num);
-	free(client);
+	fifo_buffer_free(client->input);
+
+	g_log(G_LOG_DOMAIN, LOG_LEVEL_SECURE,
+	      "[%u] closed", client->num);
+	g_free(client);
 }
 
-static const char *
-sockaddr_to_tmp_string(const struct sockaddr *addr)
-{
-	const char *hostname;
-
-	switch (addr->sa_family) {
-#ifdef HAVE_TCP
-	case AF_INET:
-		hostname = (const char *)inet_ntoa(((const struct sockaddr_in *)
-						    addr)->sin_addr);
-		if (!hostname)
-			hostname = "error getting ipv4 address";
-		break;
-#ifdef HAVE_IPV6
-	case AF_INET6:
-		{
-			static char host[INET6_ADDRSTRLEN + 1];
-			memset(host, 0, INET6_ADDRSTRLEN + 1);
-			if (inet_ntop(AF_INET6, (const void *)
-				      &(((const struct sockaddr_in6 *)addr)->
-					sin6_addr), host,
-				      INET6_ADDRSTRLEN)) {
-				hostname = (const char *)host;
-			} else {
-				hostname = "error getting ipv6 address";
-			}
-		}
-		break;
-#endif
-#endif /* HAVE_TCP */
-#ifdef HAVE_UN
-	case AF_UNIX:
-		hostname = "local connection";
-		break;
-#endif /* HAVE_UN */
-	default:
-		hostname = "unknown";
-	}
-
-	return hostname;
-}
-
-void client_new(int fd, const struct sockaddr *addr, int uid)
+void client_new(int fd, const struct sockaddr *sa, size_t sa_length, int uid)
 {
 	struct client *client;
+	char *remote;
 
 	if (num_clients >= client_max_connections) {
-		ERROR("Max Connections Reached!\n");
-		xclose(fd);
+		g_warning("Max Connections Reached!");
+		close(fd);
 		return;
 	}
 
-	client = xcalloc(1, sizeof(*client));
-	list_add(&client->siblings, &clients);
+	client = g_new0(struct client, 1);
+	clients = g_list_prepend(clients, client);
 	++num_clients;
+
 	client_init(client, fd);
 	client->uid = uid;
-	SECURE("client %i: opened from %s\n", client->num,
-	       sockaddr_to_tmp_string(addr));
+
+	client->last_activity = g_timer_new();
+
+	remote = sockaddr_to_string(sa, sa_length, NULL);
+	g_log(G_LOG_DOMAIN, LOG_LEVEL_SECURE,
+	      "[%u] opened from %s", client->num, remote);
+	g_free(remote);
 }
 
 static int client_process_line(struct client *client, char *line)
@@ -277,15 +304,15 @@ static int client_process_line(struct client *client, char *line)
 	} else if (client->idle_waiting) {
 		/* during idle mode, clients must not send anything
 		   except "noidle" */
-		ERROR("client %i: command \"%s\" during idle\n",
-		      client->num, line);
+		g_warning("[%u] command \"%s\" during idle",
+			  client->num, line);
 		return COMMAND_RETURN_CLOSE;
 	}
 
 	if (client->cmd_list_OK >= 0) {
 		if (strcmp(line, CLIENT_LIST_MODE_END) == 0) {
-			DEBUG("client %i: process command "
-			      "list\n", client->num);
+			g_debug("[%u] process command list",
+				client->num);
 
 			/* for scalability reasons, we have prepended
 			   each new command; now we have to reverse it
@@ -295,8 +322,8 @@ static int client_process_line(struct client *client, char *line)
 			ret = command_process_list(client,
 						   client->cmd_list_OK,
 						   client->cmd_list);
-			DEBUG("client %i: process command "
-			      "list returned %i\n", client->num, ret);
+			g_debug("[%u] process command "
+				"list returned %i", client->num, ret);
 
 			if (ret == COMMAND_RETURN_CLOSE ||
 			    client_is_expired(client))
@@ -314,14 +341,11 @@ static int client_process_line(struct client *client, char *line)
 			client->cmd_list_size += len;
 			if (client->cmd_list_size >
 			    client_max_command_list_size) {
-				ERROR("client %i: command "
-				      "list size (%lu) is "
-				      "larger than the max "
-				      "(%lu)\n",
-				      client->num,
-				      (unsigned long)client->cmd_list_size,
-				      (unsigned long)
-				      client_max_command_list_size);
+				g_warning("[%u] command list size (%lu) "
+					  "is larger than the max (%lu)",
+					  client->num,
+					  (unsigned long)client->cmd_list_size,
+					  (unsigned long)client_max_command_list_size);
 				return COMMAND_RETURN_CLOSE;
 			} else
 				new_cmd_list_ptr(client, line);
@@ -334,11 +358,11 @@ static int client_process_line(struct client *client, char *line)
 			client->cmd_list_OK = 1;
 			ret = 1;
 		} else {
-			DEBUG("client %i: process command \"%s\"\n",
-			      client->num, line);
+			g_debug("[%u] process command \"%s\"",
+				client->num, line);
 			ret = command_process(client, line);
-			DEBUG("client %i: command returned %i\n",
-			      client->num, ret);
+			g_debug("[%u] command returned %i",
+				client->num, ret);
 
 			if (ret == COMMAND_RETURN_CLOSE ||
 			    client_is_expired(client))
@@ -354,54 +378,45 @@ static int client_process_line(struct client *client, char *line)
 	return ret;
 }
 
+static char *
+client_read_line(struct client *client)
+{
+	const char *p, *newline;
+	size_t length;
+	char *line;
+
+	p = fifo_buffer_read(client->input, &length);
+	if (p == NULL)
+		return NULL;
+
+	newline = memchr(p, '\n', length);
+	if (newline == NULL)
+		return NULL;
+
+	line = g_strndup(p, newline - p);
+	fifo_buffer_consume(client->input, newline - p + 1);
+
+	return g_strchomp(line);
+}
+
 static int client_input_received(struct client *client, size_t bytesRead)
 {
-	char *start = client->buffer + client->bufferPos, *end;
-	char *newline, *next;
+	char *line;
 	int ret;
 
-	assert(client->bufferPos <= client->bufferLength);
-	assert(client->bufferLength + bytesRead <= sizeof(client->buffer));
-
-	client->bufferLength += bytesRead;
-	end = client->buffer + client->bufferLength;
+	fifo_buffer_append(client->input, bytesRead);
 
 	/* process all lines */
-	while ((newline = memchr(start, '\n', end - start)) != NULL) {
-		next = newline + 1;
 
-		if (newline > start && newline[-1] == '\r')
-			--newline;
-		*newline = 0;
+	while ((line = client_read_line(client)) != NULL) {
+		ret = client_process_line(client, line);
+		g_free(line);
 
-		ret = client_process_line(client, start);
 		if (ret == COMMAND_RETURN_KILL ||
 		    ret == COMMAND_RETURN_CLOSE)
 			return ret;
 		if (client_is_expired(client))
 			return COMMAND_RETURN_CLOSE;
-
-		start = next;
-	}
-
-	/* mark consumed lines */
-	client->bufferPos = start - client->buffer;
-
-	/* if we're have reached the buffer's end, close the gab at
-	   the beginning */
-	if (client->bufferLength == sizeof(client->buffer)) {
-		if (client->bufferPos == 0) {
-			ERROR("client %i: buffer overflow\n",
-			      client->num);
-			return COMMAND_RETURN_CLOSE;
-		}
-		assert(client->bufferLength >= client->bufferPos
-		       && "bufferLength >= bufferPos");
-		client->bufferLength -= client->bufferPos;
-		memmove(client->buffer,
-			client->buffer + client->bufferPos,
-			client->bufferLength);
-		client->bufferPos = 0;
 	}
 
 	return 0;
@@ -409,169 +424,159 @@ static int client_input_received(struct client *client, size_t bytesRead)
 
 static int client_read(struct client *client)
 {
-	ssize_t bytesRead;
+	char *p;
+	size_t max_length;
+	GError *error = NULL;
+	GIOStatus status;
+	gsize bytes_read;
 
-	assert(client->bufferPos <= client->bufferLength);
-	assert(client->bufferLength < sizeof(client->buffer));
+	assert(client != NULL);
+	assert(client->channel != NULL);
 
-	bytesRead = read(client->fd,
-			 client->buffer + client->bufferLength,
-			 sizeof(client->buffer) - client->bufferLength);
+	p = fifo_buffer_write(client->input, &max_length);
+	if (p == NULL) {
+		g_warning("[%u] buffer overflow", client->num);
+		return COMMAND_RETURN_CLOSE;
+	}
 
-	if (bytesRead > 0)
-		return client_input_received(client, bytesRead);
-	else if (bytesRead < 0 && errno == EINTR)
+	status = g_io_channel_read_chars(client->channel, p, max_length,
+					 &bytes_read, &error);
+	switch (status) {
+	case G_IO_STATUS_NORMAL:
+		return client_input_received(client, bytes_read);
+
+	case G_IO_STATUS_AGAIN:
 		/* try again later, after select() */
 		return 0;
-	else
-		/* peer disconnected or I/O error */
+
+	case G_IO_STATUS_EOF:
+		/* peer disconnected */
 		return COMMAND_RETURN_CLOSE;
-}
 
-static void client_manager_register_read_fd(fd_set * fds, int *fdmax)
-{
-	struct client *client;
-
-	FD_ZERO(fds);
-	addListenSocketsToFdSet(fds, fdmax);
-
-	list_for_each_entry(client, &clients, siblings) {
-		if (!client_is_expired(client) &&
-		    g_queue_is_empty(client->deferred_send)) {
-			FD_SET(client->fd, fds);
-			if (*fdmax < client->fd)
-				*fdmax = client->fd;
-		}
+	case G_IO_STATUS_ERROR:
+		/* I/O error */
+		g_warning("failed to read from client %d: %s",
+			  client->num, error->message);
+		g_error_free(error);
+		return COMMAND_RETURN_CLOSE;
 	}
+
+	/* unreachable */
+	return COMMAND_RETURN_CLOSE;
 }
 
-static void client_manager_register_write_fd(fd_set * fds, int *fdmax)
+static gboolean
+client_out_event(G_GNUC_UNUSED GIOChannel *source, GIOCondition condition,
+		 gpointer data);
+
+static gboolean
+client_in_event(G_GNUC_UNUSED GIOChannel *source,
+		GIOCondition condition,
+		gpointer data)
 {
-	struct client *client;
-
-	FD_ZERO(fds);
-
-	list_for_each_entry(client, &clients, siblings) {
-		if (client->fd >= 0 && !client_is_expired(client)
-		    && !g_queue_is_empty(client->deferred_send)) {
-			FD_SET(client->fd, fds);
-			if (*fdmax < client->fd)
-				*fdmax = client->fd;
-		}
-	}
-}
-
-int client_manager_io(void)
-{
-	fd_set rfds;
-	fd_set wfds;
-	fd_set efds;
-	struct client *client, *n;
+	struct client *client = data;
 	int ret;
-	int fdmax = 0;
 
-	FD_ZERO( &efds );
-	client_manager_register_read_fd(&rfds, &fdmax);
-	client_manager_register_write_fd(&wfds, &fdmax);
+	assert(!client_is_expired(client));
 
-	registered_IO_add_fds(&fdmax, &rfds, &wfds, &efds);
-
-	main_notify_lock();
-	ret = select(fdmax + 1, &rfds, &wfds, &efds, NULL);
-	main_notify_unlock();
-
-	if (ret < 0) {
-		if (errno == EINTR)
-			return 0;
-
-		FATAL("select() failed: %s\n", strerror(errno));
+	if (condition != G_IO_IN) {
+		client_set_expired(client);
+		return false;
 	}
 
-	registered_IO_consume_fds(&ret, &rfds, &wfds, &efds);
+	g_timer_start(client->last_activity);
 
-	getConnections(&rfds);
+	ret = client_read(client);
+	switch (ret) {
+	case COMMAND_RETURN_KILL:
+		client_close(client);
+		g_main_loop_quit(main_loop);
+		return false;
 
-	list_for_each_entry_safe(client, n, &clients, siblings) {
-		if (FD_ISSET(client->fd, &rfds)) {
-			ret = client_read(client);
-			if (ret == COMMAND_RETURN_KILL)
-				return COMMAND_RETURN_KILL;
-			if (ret == COMMAND_RETURN_CLOSE) {
-				client_close(client);
-				continue;
-			}
-
-			assert(!client_is_expired(client));
-
-			client->lastTime = time(NULL);
-		}
-		if (!client_is_expired(client) &&
-		    FD_ISSET(client->fd, &wfds)) {
-			client_write_deferred(client);
-			client->lastTime = time(NULL);
-		}
+	case COMMAND_RETURN_CLOSE:
+		client_close(client);
+		return false;
 	}
 
-	return 0;
+	if (client_is_expired(client)) {
+		client_close(client);
+		return false;
+	}
+
+	if (!g_queue_is_empty(client->deferred_send)) {
+		/* deferred buffers exist: schedule write */
+		client->source_id = g_io_add_watch(client->channel,
+						   G_IO_OUT|G_IO_ERR|G_IO_HUP,
+						   client_out_event, client);
+		return false;
+	}
+
+	/* read more */
+	return true;
+}
+
+static gboolean
+client_out_event(G_GNUC_UNUSED GIOChannel *source, GIOCondition condition,
+		 gpointer data)
+{
+	struct client *client = data;
+
+	assert(!client_is_expired(client));
+
+	if (condition != G_IO_OUT) {
+		client_set_expired(client);
+		return false;
+	}
+
+	client_write_deferred(client);
+
+	if (client_is_expired(client)) {
+		client_close(client);
+		return false;
+	}
+
+	g_timer_start(client->last_activity);
+
+	if (g_queue_is_empty(client->deferred_send)) {
+		/* done sending deferred buffers exist: schedule
+		   read */
+		client->source_id = g_io_add_watch(client->channel,
+						   G_IO_IN|G_IO_ERR|G_IO_HUP,
+						   client_in_event, client);
+		return false;
+	}
+
+	/* write more */
+	return true;
 }
 
 void client_manager_init(void)
 {
-	char *test;
-	ConfigParam *param;
+	client_timeout = config_get_positive(CONF_CONN_TIMEOUT,
+					     CLIENT_TIMEOUT_DEFAULT);
+	client_max_connections =
+		config_get_positive(CONF_MAX_CONN,
+				    CLIENT_MAX_CONNECTIONS_DEFAULT);
+	client_max_command_list_size =
+		config_get_positive(CONF_MAX_COMMAND_LIST_SIZE,
+				    CLIENT_MAX_COMMAND_LIST_DEFAULT / 1024)
+		* 1024;
 
-	param = getConfigParam(CONF_CONN_TIMEOUT);
-
-	if (param) {
-		client_timeout = strtol(param->value, &test, 10);
-		if (*test != '\0' || client_timeout <= 0) {
-			FATAL("connection timeout \"%s\" is not a positive "
-			      "integer, line %i\n", CONF_CONN_TIMEOUT,
-			      param->line);
-		}
-	}
-
-	param = getConfigParam(CONF_MAX_CONN);
-
-	if (param) {
-		client_max_connections = strtol(param->value, &test, 10);
-		if (*test != '\0' || client_max_connections <= 0) {
-			FATAL("max connections \"%s\" is not a positive integer"
-			      ", line %i\n", param->value, param->line);
-		}
-	} else
-		client_max_connections = CLIENT_MAX_CONNECTIONS_DEFAULT;
-
-	param = getConfigParam(CONF_MAX_COMMAND_LIST_SIZE);
-
-	if (param) {
-		long tmp = strtol(param->value, &test, 10);
-		if (*test != '\0' || tmp <= 0) {
-			FATAL("max command list size \"%s\" is not a positive "
-			      "integer, line %i\n", param->value, param->line);
-		}
-		client_max_command_list_size = tmp * 1024;
-	}
-
-	param = getConfigParam(CONF_MAX_OUTPUT_BUFFER_SIZE);
-
-	if (param) {
-		long tmp = strtol(param->value, &test, 10);
-		if (*test != '\0' || tmp <= 0) {
-			FATAL("max output buffer size \"%s\" is not a positive "
-			      "integer, line %i\n", param->value, param->line);
-		}
-		client_max_output_buffer_size = tmp * 1024;
-	}
+	client_max_output_buffer_size =
+		config_get_positive(CONF_MAX_OUTPUT_BUFFER_SIZE,
+				    CLIENT_MAX_OUTPUT_BUFFER_SIZE_DEFAULT / 1024)
+		* 1024;
 }
 
 static void client_close_all(void)
 {
-	struct client *client, *n;
+	while (clients != NULL) {
+		struct client *client = clients->data;
 
-	list_for_each_entry_safe(client, n, &clients, siblings)
 		client_close(client);
-	num_clients = 0;
+	}
+
+	assert(num_clients == 0);
 }
 
 void client_manager_deinit(void)
@@ -579,29 +584,79 @@ void client_manager_deinit(void)
 	client_close_all();
 
 	client_max_connections = 0;
+
+	if (expire_source_id != 0)
+		g_source_remove(expire_source_id);
 }
 
-void client_manager_expire(void)
+static void
+client_check_expired_callback(gpointer data, G_GNUC_UNUSED gpointer user_data)
 {
-	struct client *client, *n;
+	struct client *client = data;
 
-	list_for_each_entry_safe(client, n, &clients, siblings) {
-		if (client_is_expired(client)) {
-			DEBUG("client %i: expired\n", client->num);
-			client_close(client);
-		} else if (!client->idle_waiting && /* idle clients
-						       never expire */
-			   time(NULL) - client->lastTime >
-			   client_timeout) {
-			DEBUG("client %i: timeout\n", client->num);
-			client_close(client);
-		}
+	if (client_is_expired(client)) {
+		g_debug("[%u] expired", client->num);
+		client_close(client);
+	} else if (!client->idle_waiting && /* idle clients
+					       never expire */
+		   (int)g_timer_elapsed(client->last_activity, NULL) >
+		   client_timeout) {
+		g_debug("[%u] timeout", client->num);
+		client_close(client);
 	}
 }
 
+static void
+client_manager_expire(void)
+{
+	g_list_foreach(clients, client_check_expired_callback, NULL);
+}
+
+static size_t
+client_write_deferred_buffer(struct client *client,
+			     const struct deferred_buffer *buffer)
+{
+	GError *error = NULL;
+	GIOStatus status;
+	gsize bytes_written;
+
+	assert(client != NULL);
+	assert(client->channel != NULL);
+	assert(buffer != NULL);
+
+	status = g_io_channel_write_chars
+		(client->channel, buffer->data, buffer->size,
+		 &bytes_written, &error);
+	switch (status) {
+	case G_IO_STATUS_NORMAL:
+		return bytes_written;
+
+	case G_IO_STATUS_AGAIN:
+		return 0;
+
+	case G_IO_STATUS_EOF:
+		/* client has disconnected */
+
+		client_set_expired(client);
+		return 0;
+
+	case G_IO_STATUS_ERROR:
+		/* I/O error */
+
+		client_set_expired(client);
+		g_warning("failed to flush buffer for %i: %s",
+			  client->num, error->message);
+		g_error_free(error);
+		return 0;
+	}
+
+	/* unreachable */
+	return 0;
+}
+
 static void client_write_deferred(struct client *client)
 {
-	ssize_t ret = 0;
+	size_t ret;
 
 	while (!g_queue_is_empty(client->deferred_send)) {
 		struct deferred_buffer *buf =
@@ -610,10 +665,11 @@ static void client_write_deferred(struct client *client)
 		assert(buf->size > 0);
 		assert(buf->size <= client->deferred_bytes);
 
-		ret = write(client->fd, buf->data, buf->size);
-		if (ret < 0)
+		ret = client_write_deferred_buffer(client, buf);
+		if (ret == 0)
 			break;
-		else if ((size_t)ret < buf->size) {
+
+		if (ret < buf->size) {
 			assert(client->deferred_bytes >= (size_t)ret);
 			client->deferred_bytes -= ret;
 			buf->size -= ret;
@@ -625,21 +681,17 @@ static void client_write_deferred(struct client *client)
 
 			assert(client->deferred_bytes >= decr);
 			client->deferred_bytes -= decr;
-			free(buf);
+			g_free(buf);
 			g_queue_pop_head(client->deferred_send);
 		}
-		client->lastTime = time(NULL);
+
+		g_timer_start(client->last_activity);
 	}
 
 	if (g_queue_is_empty(client->deferred_send)) {
-		DEBUG("client %i: buffer empty %lu\n", client->num,
-		      (unsigned long)client->deferred_bytes);
+		g_debug("[%u] buffer empty %lu", client->num,
+			(unsigned long)client->deferred_bytes);
 		assert(client->deferred_bytes == 0);
-	} else if (ret < 0 && errno != EAGAIN && errno != EINTR) {
-		/* cause client to close */
-		DEBUG("client %i: problems flushing buffer\n",
-		      client->num);
-		client_set_expired(client);
 	}
 }
 
@@ -654,11 +706,11 @@ static void client_defer_output(struct client *client,
 	alloc = sizeof(*buf) - sizeof(buf->data) + length;
 	client->deferred_bytes += alloc;
 	if (client->deferred_bytes > client_max_output_buffer_size) {
-		ERROR("client %i: output buffer size (%lu) is "
-		      "larger than the max (%lu)\n",
-		      client->num,
-		      (unsigned long)client->deferred_bytes,
-		      (unsigned long)client_max_output_buffer_size);
+		g_warning("[%u] output buffer size (%lu) is "
+			  "larger than the max (%lu)",
+			  client->num,
+			  (unsigned long)client->deferred_bytes,
+			  (unsigned long)client_max_output_buffer_size);
 		/* cause client to close */
 		client_set_expired(client);
 		return;
@@ -674,25 +726,45 @@ static void client_defer_output(struct client *client,
 static void client_write_direct(struct client *client,
 				const char *data, size_t length)
 {
-	ssize_t ret;
+	GError *error = NULL;
+	GIOStatus status;
+	gsize bytes_written;
 
+	assert(client != NULL);
+	assert(client->channel != NULL);
+	assert(data != NULL);
 	assert(length > 0);
 	assert(g_queue_is_empty(client->deferred_send));
 
-	if ((ret = write(client->fd, data, length)) < 0) {
-		if (errno == EAGAIN || errno == EINTR) {
-			client_defer_output(client, data, length);
-		} else {
-			DEBUG("client %i: problems writing\n", client->num);
-			client_set_expired(client);
-			return;
-		}
-	} else if ((size_t)ret < client->send_buf_used) {
-		client_defer_output(client, data + ret, length - ret);
+	status = g_io_channel_write_chars(client->channel, data, length,
+					  &bytes_written, &error);
+	switch (status) {
+	case G_IO_STATUS_NORMAL:
+	case G_IO_STATUS_AGAIN:
+		break;
+
+	case G_IO_STATUS_EOF:
+		/* client has disconnected */
+
+		client_set_expired(client);
+		return;
+
+	case G_IO_STATUS_ERROR:
+		/* I/O error */
+
+		client_set_expired(client);
+		g_warning("failed to write to %i: %s",
+			  client->num, error->message);
+		g_error_free(error);
+		return;
 	}
 
+	if (bytes_written < length)
+		client_defer_output(client, data + bytes_written,
+				    length - bytes_written);
+
 	if (!g_queue_is_empty(client->deferred_send))
-		DEBUG("client %i: buffer created\n", client->num);
+		g_debug("[%u] buffer created", client->num);
 }
 
 static void client_write_output(struct client *client)
@@ -704,6 +776,9 @@ static void client_write_output(struct client *client)
 		client_defer_output(client, client->send_buf,
 				    client->send_buf_used);
 
+		if (client_is_expired(client))
+			return;
+
 		/* try to flush the deferred buffers now; the current
 		   server command may take too long to finish, and
 		   meanwhile try to feed output to the client,
@@ -719,7 +794,10 @@ static void client_write_output(struct client *client)
 	client->send_buf_used = 0;
 }
 
-void client_write(struct client *client, const char *buffer, size_t buflen)
+/**
+ * Write a block of data to the client.
+ */
+static void client_write(struct client *client, const char *buffer, size_t buflen)
 {
 	/* if the client is going to be closed, do nothing */
 	if (client_is_expired(client))
@@ -763,10 +841,10 @@ void client_vprintf(struct client *client, const char *fmt, va_list args)
 		/* wtf.. */
 		return;
 
-	buffer = xmalloc(length + 1);
+	buffer = g_malloc(length + 1);
 	vsnprintf(buffer, length + 1, fmt, args);
 	client_write(client, buffer, length);
-	free(buffer);
+	g_free(buffer);
 }
 
 G_GNUC_PRINTF(2, 3) void client_printf(struct client *client, const char *fmt, ...)
@@ -802,26 +880,31 @@ client_idle_notify(struct client *client)
 	}
 
 	client_puts(client, "OK\n");
-	client->lastTime = time(NULL);
+	g_timer_start(client->last_activity);
+}
+
+static void
+client_idle_callback(gpointer data, gpointer user_data)
+{
+	struct client *client = data;
+	unsigned flags = GPOINTER_TO_UINT(user_data);
+
+	if (client_is_expired(client))
+		return;
+
+	client->idle_flags |= flags;
+	if (client->idle_waiting
+	    && (client->idle_flags & client->idle_subscriptions)) {
+		client_idle_notify(client);
+		client_write_output(client);
+	}
 }
 
 void client_manager_idle_add(unsigned flags)
 {
-	struct client *client;
-
 	assert(flags != 0);
 
-	list_for_each_entry(client, &clients, siblings) {
-		if (client_is_expired(client))
-			continue;
-
-		client->idle_flags |= flags;
-		if (client->idle_waiting
-		    && (client->idle_flags & client->idle_subscriptions)) {
-			client_idle_notify(client);
-			client_write_output(client);
-		}
-	}
+	g_list_foreach(clients, client_idle_callback, GUINT_TO_POINTER(flags));
 }
 
 bool client_idle_wait(struct client *client, unsigned flags)
