@@ -299,10 +299,10 @@ flac_cue_tag_load(const char *file)
 	unsigned int sample_rate = 0;
 	FLAC__uint64 track_time = 0;
 #ifdef HAVE_CUE /* libcue */
-	FLAC__StreamMetadata* vc = FLAC__metadata_object_new(FLAC__METADATA_TYPE_VORBIS_COMMENT);
+	FLAC__StreamMetadata* vc;
 #endif /* libcue */
 	FLAC__StreamMetadata* si = FLAC__metadata_object_new(FLAC__METADATA_TYPE_STREAMINFO);
-	FLAC__StreamMetadata* cs = FLAC__metadata_object_new(FLAC__METADATA_TYPE_CUESHEET);
+	FLAC__StreamMetadata* cs;
 
 	tnum = flac_vtrack_tnum(file);
 	char_tnum = g_strdup_printf("%u", tnum);
@@ -326,6 +326,7 @@ flac_cue_tag_load(const char *file)
 				}
 			}
 		}
+
 		FLAC__metadata_object_delete(vc);
 	}
 #endif /* libcue */
@@ -628,21 +629,15 @@ flac_container_decode(struct decoder* decoder,
 			FLAC__uint64 seek_sample = t_start +
 				(decoder_seek_where(decoder) * data.audio_format.sample_rate);
 
-			//if (seek_sample >= t_start && seek_sample <= t_end && data.total_time > 30)
-			if (seek_sample >= t_start && seek_sample <= t_end)
-			{
-				if (flac_seek_absolute(flac_dec, (FLAC__uint64)seek_sample))
-				{
-					data.time = (float)(seek_sample - t_start) /
-					    data.audio_format.sample_rate;
-					data.position = 0;
+			if (seek_sample >= t_start && seek_sample <= t_end &&
+			    flac_seek_absolute(flac_dec, (FLAC__uint64)seek_sample)) {
+				data.time = (float)(seek_sample - t_start) /
+					data.audio_format.sample_rate;
+				data.position = 0;
 
-					decoder_command_finished(decoder);
-				}
-				else
-					decoder_seek_error(decoder);
-					//decoder_command_finished(decoder);
-			}
+				decoder_command_finished(decoder);
+			} else
+				decoder_seek_error(decoder);
 		}
 		else if (flac_get_state(flac_dec) == flac_decoder_eof)
 			break;
@@ -876,9 +871,11 @@ oggflac_decode(struct decoder *decoder, struct input_stream *input_stream)
 
 static const char *const oggflac_suffixes[] = { "ogg", "oga", NULL };
 static const char *const oggflac_mime_types[] = {
-	"audio/x-flac+ogg",
 	"application/ogg",
 	"application/x-ogg",
+	"audio/ogg",
+	"audio/x-flac+ogg",
+	"audio/x-ogg",
 	NULL
 };
 
@@ -899,7 +896,11 @@ const struct decoder_plugin oggflac_decoder_plugin = {
 
 static const char *const flac_suffixes[] = { "flac", NULL };
 static const char *const flac_mime_types[] = {
-	"audio/x-flac", "application/x-flac", NULL
+	"application/flac",
+	"application/x-flac",
+	"audio/flac",
+	"audio/x-flac",
+	NULL
 };
 
 const struct decoder_plugin flac_decoder_plugin = {
