@@ -72,7 +72,7 @@ format_samples_int(int bytes_per_sample, void *buffer, uint32_t count)
 
 	switch (bytes_per_sample) {
 	case 1: {
-		uchar *dst = buffer;
+		int8_t *dst = buffer;
 		/*
 		 * The asserts like the following one are because we do the
 		 * formatting of samples within a single buffer. The size
@@ -185,10 +185,9 @@ wavpack_decode(struct decoder *decoder, WavpackContext *wpc, bool can_seek,
 	do {
 		if (decoder_get_command(decoder) == DECODE_COMMAND_SEEK) {
 			if (can_seek) {
-				int where;
+				unsigned where = decoder_seek_where(decoder) *
+					audio_format.sample_rate;
 
-				where = decoder_seek_where(decoder);
-				where *= audio_format.sample_rate;
 				if (WavpackSeekSample(wpc, where)) {
 					position = where;
 					decoder_command_finished(decoder);
