@@ -1,6 +1,6 @@
-/* the Music Player Daemon (MPD)
- * Copyright (C) 2003-2007 by Warren Dukes (warren.dukes@gmail.com)
- * This project's homepage is: http://www.musicpd.org
+/*
+ * Copyright (C) 2003-2009 The Music Player Daemon Project
+ * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -11,9 +11,10 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #ifndef MPD_DIRECTORY_H
@@ -26,18 +27,15 @@
 #include <sys/types.h>
 
 #define DIRECTORY_DIR		"directory: "
-#define DIRECTORY_MTIME		"mtime: " /* DEPRECATED, noop-read-only */
-#define DIRECTORY_BEGIN		"begin: "
-#define DIRECTORY_END		"end: "
-#define DIRECTORY_INFO_BEGIN	"info_begin"
-#define DIRECTORY_INFO_END	"info_end"
-#define DIRECTORY_MPD_VERSION	"mpd_version: "
-#define DIRECTORY_FS_CHARSET	"fs_charset: "
+
+#define DEVICE_INARCHIVE	(unsigned)(-1)
+#define DEVICE_CONTAINER	(unsigned)(-2)
 
 struct directory {
 	struct dirvec children;
 	struct songvec songs;
 	struct directory *parent;
+	time_t mtime;
 	ino_t inode;
 	dev_t device;
 	unsigned stat; /* not needed if ino_t == dev_t == 0 is impossible */
@@ -68,6 +66,15 @@ directory_get_path(const struct directory *directory)
 	return directory->path;
 }
 
+/**
+ * Is this the root directory of the music database?
+ */
+static inline bool
+directory_is_root(const struct directory *directory)
+{
+	return directory->parent == NULL;
+}
+
 /**
  * Returns the base name of the directory.
  */
@@ -91,8 +98,25 @@ directory_new_child(struct directory *directory, const char *name)
 void
 directory_prune_empty(struct directory *directory);
 
+/**
+ * Looks up a directory by its relative URI.
+ *
+ * @param directory the parent (or grandparent, ...) directory
+ * @param uri the relative URI
+ * @return the directory, or NULL if none was found
+ */
 struct directory *
-directory_get_directory(struct directory *directory, const char *name);
+directory_lookup_directory(struct directory *directory, const char *uri);
+
+/**
+ * Looks up a song by its relative URI.
+ *
+ * @param directory the parent (or grandparent, ...) directory
+ * @param uri the relative URI
+ * @return the song, or NULL if none was found
+ */
+struct song *
+directory_lookup_song(struct directory *directory, const char *uri);
 
 void
 directory_sort(struct directory *directory);
