@@ -1,6 +1,6 @@
-/* the Music Player Daemon (MPD)
- * Copyright (C) 2003-2007 by Warren Dukes (warren.dukes@gmail.com)
- * This project's homepage is: http://www.musicpd.org
+/*
+ * Copyright (C) 2003-2009 The Music Player Daemon Project
+ * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -11,9 +11,10 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include "directory_print.h"
@@ -21,7 +22,7 @@
 #include "client.h"
 #include "song_print.h"
 
-static int
+static void
 dirvec_print(struct client *client, const struct dirvec *dv)
 {
 	size_t i;
@@ -29,15 +30,11 @@ dirvec_print(struct client *client, const struct dirvec *dv)
 	for (i = 0; i < dv->nr; ++i)
 		client_printf(client, DIRECTORY_DIR "%s\n",
 			      directory_get_path(dv->base[i]));
-
-	return 0;
 }
 
-int
+void
 directory_print(struct client *client, const struct directory *directory)
 {
 	dirvec_print(client, &directory->children);
 	songvec_print(client, &directory->songs);
-
-	return 0;
 }
