@@ -20,6 +20,16 @@
 #ifndef MPD_INPUT_CURL_H
 #define MPD_INPUT_CURL_H
 
+struct input_stream;
+
 extern const struct input_plugin input_plugin_curl;
 
+/**
+ * This is a workaround for an input_stream API deficiency; after
+ * exchanging the input_stream pointer in input_rewind_open(), this
+ * function is called to reinitialize CURL's data pointers.
+ */
+void
+input_curl_reinit(struct input_stream *is);
+
 #endif
