@@ -221,14 +221,19 @@ map_spl_path(void)
 char *
 map_spl_utf8_to_fs(const char *name)
 {
-	char *filename = g_strconcat(name, PLAYLIST_FILE_SUFFIX, NULL);
-	char *path;
+	char *filename_utf8, *filename_fs, *path;
 
 	if (playlist_dir == NULL)
 		return NULL;
 
-	path = g_build_filename(playlist_dir, filename, NULL);
-	g_free(filename);
+	filename_utf8 = g_strconcat(name, PLAYLIST_FILE_SUFFIX, NULL);
+	filename_fs = utf8_to_fs_charset(filename_utf8);
+	g_free(filename_utf8);
+	if (filename_fs == NULL)
+		return NULL;
+
+	path = g_build_filename(playlist_dir, filename_fs, NULL);
+	g_free(filename_fs);
 
 	return path;
 }
