@@ -1,6 +1,6 @@
-/* the Music Player Daemon (MPD)
- * Copyright (C) 2008 Max Kellermann <max@duempel.org>
- * This project's homepage is: http://www.musicpd.org
+/*
+ * Copyright (C) 2003-2009 The Music Player Daemon Project
+ * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -11,28 +11,25 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #ifndef MPD_NOTIFY_H
 #define MPD_NOTIFY_H
 
+#include <glib.h>
+
 #include <stdbool.h>
-#include <pthread.h>
 
 struct notify {
-	pthread_mutex_t mutex;
-	pthread_cond_t cond;
+	GMutex *mutex;
+	GCond *cond;
 	bool pending;
 };
 
-#define NOTIFY_INITIALIZER { \
-	.mutex = PTHREAD_MUTEX_INITIALIZER, \
-	.cond = PTHREAD_COND_INITIALIZER, \
-}
-
 void notify_init(struct notify *notify);
 
 void notify_deinit(struct notify *notify);
