@@ -1,6 +1,6 @@
-/* the Music Player Daemon (MPD)
- * Copyright (C) 2008 Max Kellermann <max@duempel.org>
- * This project's homepage is: http://www.musicpd.org
+/*
+ * Copyright (C) 2003-2009 The Music Player Daemon Project
+ * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -11,19 +11,24 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #ifndef MPD_OUTPUT_LIST_H
 #define MPD_OUTPUT_LIST_H
 
+#include <stdio.h>
+
 extern const struct audio_output_plugin *audio_output_plugins[];
 
 const struct audio_output_plugin *
 audio_output_plugin_get(const char *name);
 
+void audio_output_plugin_print_all_types(FILE * fp);
+
 #define audio_output_plugins_for_each(plugin, i) \
 	for (i = 0; (plugin = audio_output_plugins[i]) != NULL; ++i)
 
