@@ -248,6 +248,15 @@ static gpointer audio_output_task(gpointer arg)
 			break;
 
 		case AO_COMMAND_PAUSE:
+			if (!ao->open) {
+				/* the output has failed after
+				   audio_output_all_pause() has
+				   submitted the PAUSE command; bail
+				   out */
+				ao_command_finished(ao);
+				break;
+			}
+
 			ao_pause(ao);
 			/* don't "break" here: this might cause
 			   ao_play() to be called when command==CLOSE
@@ -257,7 +266,8 @@ static gpointer audio_output_task(gpointer arg)
 
 		case AO_COMMAND_CANCEL:
 			ao->chunk = NULL;
-			ao_plugin_cancel(ao->plugin, ao->data);
+			if (ao->open)
+				ao_plugin_cancel(ao->plugin, ao->data);
 			ao_command_finished(ao);
 
 			/* the player thread will now clear our music
