@@ -1,6 +1,6 @@
-/* the Music Player Daemon (MPD)
- * Copyright (C) 2008 Max Kellermann <max@duempel.org>
- * This project's homepage is: http://www.musicpd.org
+/*
+ * Copyright (C) 2003-2009 The Music Player Daemon Project
+ * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -11,9 +11,10 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #ifndef MPD_PCM_DITHER_H
@@ -21,20 +22,25 @@
 
 #include <stdint.h>
 
-struct pcm_dither_24 {
+struct pcm_dither {
 	int32_t error[3];
 	int32_t random;
 };
 
 static inline void
-pcm_dither_24_init(struct pcm_dither_24 *dither)
+pcm_dither_24_init(struct pcm_dither *dither)
 {
 	dither->error[0] = dither->error[1] = dither->error[2] = 0;
 	dither->random = 0;
 }
 
 void
-pcm_dither_24_to_16(struct pcm_dither_24 *dither,
+pcm_dither_24_to_16(struct pcm_dither *dither,
+		    int16_t *dest, const int32_t *src,
+		    unsigned num_samples);
+
+void
+pcm_dither_32_to_16(struct pcm_dither *dither,
 		    int16_t *dest, const int32_t *src,
 		    unsigned num_samples);
 
