@@ -1,6 +1,6 @@
-/* the Music Player Daemon (MPD)
- * Copyright (C) 2003-2007 by Warren Dukes (warren.dukes@gmail.com)
- * This project's homepage is: http://www.musicpd.org
+/*
+ * Copyright (C) 2003-2009 The Music Player Daemon Project
+ * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -11,22 +11,21 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include "permission.h"
-
 #include "conf.h"
-#include "log.h"
-#include "utils.h"
-#include "os_compat.h"
 
 #include <glib.h>
-#include <stdbool.h>
 
-#define PERMISSION_PASSWORD_CHAR	"@"
+#include <stdbool.h>
+#include <string.h>
+
+#define PERMISSION_PASSWORD_CHAR	'@'
 #define PERMISSION_SEPERATOR		","
 
 #define PERMISSION_READ_STRING		"read"
@@ -38,17 +37,18 @@ static GHashTable *permission_passwords;
 
 static unsigned permission_default;
 
-static unsigned parsePermissions(char *string)
+static unsigned parsePermissions(const char *string)
 {
 	unsigned permission = 0;
-	char *temp;
-	char *tok;
+	gchar **tokens;
 
 	if (!string)
 		return 0;
 
-	temp = strtok_r(string, PERMISSION_SEPERATOR, &tok);
-	while (temp) {
+	tokens = g_strsplit(string, PERMISSION_SEPERATOR, 0);
+	for (unsigned i = 0; tokens[i] != NULL; ++i) {
+		char *temp = tokens[i];
+
 		if (strcmp(temp, PERMISSION_READ_STRING) == 0) {
 			permission |= PERMISSION_READ;
 		} else if (strcmp(temp, PERMISSION_ADD_STRING) == 0) {
@@ -58,22 +58,20 @@ static unsigned parsePermissions(char *string)
 		} else if (strcmp(temp, PERMISSION_ADMIN_STRING) == 0) {
 			permission |= PERMISSION_ADMIN;
 		} else {
-			FATAL("unknown permission \"%s\"\n", temp);
+			g_error("unknown permission \"%s\"", temp);
 		}
-
-		temp = strtok_r(NULL, PERMISSION_SEPERATOR, &tok);
 	}
 
+	g_strfreev(tokens);
+
 	return permission;
 }
 
 void initPermissions(void)
 {
-	char *temp;
-	char *cp2;
 	char *password;
 	unsigned permission;
-	ConfigParam *param;
+	const struct config_param *param;
 
 	permission_passwords = g_hash_table_new_full(g_str_hash, g_str_equal,
 						     g_free, NULL);
@@ -81,37 +79,33 @@ void initPermissions(void)
 	permission_default = PERMISSION_READ | PERMISSION_ADD |
 	    PERMISSION_CONTROL | PERMISSION_ADMIN;
 
-	param = getNextConfigParam(CONF_PASSWORD, NULL);
+	param = config_get_next_param(CONF_PASSWORD, NULL);
 
 	if (param) {
 		permission_default = 0;
 
 		do {
-			if (!strstr(param->value, PERMISSION_PASSWORD_CHAR)) {
-				FATAL("\"%s\" not found in password string "
-				      "\"%s\", line %i\n",
-				      PERMISSION_PASSWORD_CHAR,
-				      param->value, param->line);
-			}
+			const char *separator =
+				strchr(param->value, PERMISSION_PASSWORD_CHAR);
 
-			if (!(temp = strtok_r(param->value,
-					      PERMISSION_PASSWORD_CHAR,
-					      &cp2))) {
-				FATAL("something weird just happened in permission.c\n");
-			}
+			if (separator == NULL)
+				g_error("\"%c\" not found in password string "
+					"\"%s\", line %i",
+					PERMISSION_PASSWORD_CHAR,
+					param->value, param->line);
 
-			password = temp;
+			password = g_strndup(param->value,
+					     separator - param->value);
 
-			permission =
-			    parsePermissions(strtok_r(NULL, "", &cp2));
+			permission = parsePermissions(separator + 1);
 
 			g_hash_table_replace(permission_passwords,
-					     g_strdup(password),
+					     password,
 					     GINT_TO_POINTER(permission));
-		} while ((param = getNextConfigParam(CONF_PASSWORD, param)));
+		} while ((param = config_get_next_param(CONF_PASSWORD, param)));
 	}
 
-	param = getConfigParam(CONF_DEFAULT_PERMS);
+	param = config_get_param(CONF_DEFAULT_PERMS);
 
 	if (param)
 		permission_default = parsePermissions(param->value);
