@@ -459,20 +459,20 @@ update_container_file(	struct directory* directory,
 	while ((vtrack = plugin->container_scan(pathname, ++tnum)) != NULL)
 	{
 		struct song* song = song_file_new(vtrack, contdir);
-		if (song == NULL)
-			return true;
+		char *child_path_fs;
 
 		// shouldn't be necessary but it's there..
 		song->mtime = st->st_mtime;
 
-		song->tag = plugin->tag_dup(map_directory_child_fs(contdir, vtrack));
+		child_path_fs = map_directory_child_fs(contdir, vtrack);
+		g_free(vtrack);
+
+		song->tag = plugin->tag_dup(child_path_fs);
+		g_free(child_path_fs);
 
 		songvec_add(&contdir->songs, song);
-		song = NULL;
 
 		modified = true;
-
-		g_free(vtrack);
 	}
 
 	g_free(pathname);
@@ -672,7 +672,11 @@ updateDirectory(struct directory *directory, const struct stat *st)
 			continue;
 
 		utf8 = fs_charset_to_utf8(ent->d_name);
-		if (utf8 == NULL || skip_symlink(directory, utf8)) {
+		if (utf8 == NULL)
+			continue;
+
+		if (skip_symlink(directory, utf8)) {
+			delete_name_in(directory, utf8);
 			g_free(utf8);
 			continue;
 		}
